using System;
using System.Collections;
using Server.Mobiles;
using Server.Items;
using Server.Targeting;

namespace Server.Mobiles
{
	[CorpseName( "an ant lion corpse" )]
	public class AntLion : BaseCreature
	{
		private Item m_hole1;
		private Item m_hole2;
		
		[Constructable]
		public AntLion() : base( AIType.AI_Melee, FightMode.Closest, 10, 1, 0.2, 0.4 )
		{
			Name = "an ant lion";
			Body = 787;
			BaseSoundID = 1006;

			SetStr( 296, 320 );
			SetDex( 81, 105 );
			SetInt( 36, 60 );

			SetHits( 151, 162 );

			SetDamage( 7, 21 );

			SetDamageType( ResistanceType.Physical, 70 );
			SetDamageType( ResistanceType.Poison, 30 );

			SetResistance( ResistanceType.Physical, 45, 60 );
			SetResistance( ResistanceType.Fire, 25, 35 );
			SetResistance( ResistanceType.Cold, 30, 40 );
			SetResistance( ResistanceType.Poison, 40, 50 );
			SetResistance( ResistanceType.Energy, 30, 35 );

			SetSkill( SkillName.MagicResist, 100.0, 120.0 );
			SetSkill( SkillName.Tactics, 90.0 );
			SetSkill( SkillName.Wrestling, 90.0 );

			Fame = 4500;
			Karma = -4500;

			VirtualArmor = 45;

			PackGem();
			PackGem();
			PackItem( new Bone() );

			int amount = Utility.RandomMinMax( 1, 10 );

			switch ( Utility.Random( 4 ) )
			{
				case 0: PackItem( new DullCopperOre( amount ) ); break;
				case 1: PackItem( new ShadowIronOre( amount ) ); break;
				case 2: PackItem( new CopperOre( amount ) ); break;
				case 3: PackItem( new BronzeOre( amount ) ); break;
			}
			
			int amt = Utility.RandomMinMax( 1, 5 );
			PackItem( new FertileDirt( amt ) );
		}
		
		public override void GenerateLoot()
		{
			AddLoot( LootPack.AosRich, 1 );
		}		

		private DateTime m_NextTunnel;
		private DateTime m_NextPoisonSpit;

		public override bool CheckMovement(Direction d, out int newZ)
		{
			//if the antlion is burrowed, he shouldn't move
			if( Frozen || Hidden )
			{
				newZ = -128;
				return false;
			}

			return base.CheckMovement (d, out newZ);
		}

		public override bool CanSee(Mobile m)
		{
			//this will stop the antlion from attacking while burrowed
			if( this.Hidden )
				return false;

			return base.CanSee (m);
		}


		public override void OnDamage( int amount, Mobile attacker, bool willKill )
		{ 
			Mobile combatant = attacker; 

			//TODO: Damage stops the antlion from burrowing

			if ( combatant == null || combatant.Deleted || combatant.Map != Map || !CanBeHarmful( combatant )  ) 
				return; 
			
			if ( DateTime.Now > m_NextTunnel ) 
			{            
				m_NextTunnel = DateTime.Now + TimeSpan.FromSeconds( 20.0 + (10.0 * Utility.RandomDouble()) ); // 5-10 seconds 
				this.GetDirectionTo( combatant.Location ); //check direction 
				this.Say("*The ant lion begins to burrow*");
				this.PlaySound( GetAngerSound() ); // Sound
				this.Freeze( TimeSpan.FromSeconds( 2 ) ); //freeze for animation 
				this.Animate( 12, 5, 1, true, false, 0 ); //animation
				new InternalTimer( this, combatant, (Point3D)this.Location ).Start(); // start timer with saved current hits value            
			}
		}

		public override void OnActionCombat() 
		{         
			Mobile combatant = Combatant; 

			if ( combatant == null || combatant.Deleted || combatant.Map != Map || !InRange( combatant, 12 ) || !CanBeHarmful( combatant ) || !InLOS( combatant ) ) 
				return; 

			if ( this.Frozen || this.Hidden )
				return;

			if ( DateTime.Now > m_NextPoisonSpit ) 
			{ 
				m_NextPoisonSpit = DateTime.Now + TimeSpan.FromSeconds( 5.0 + (10.0 * Utility.RandomDouble()) ); // 5-15 seconds 
				this.GetDirectionTo( combatant.Location ); //check direction 
				this.Freeze( TimeSpan.FromSeconds( 2 ) ); //freeze for animation 
				this.Animate( 12, 5, 1, true, false, 0 ); //animation 
				this.PlaySound( GetAngerSound() ); // Sound 
				new PoisonTimer( this, combatant, (int)this.Hits ).Start(); // start timer with saved current hits value 
			} 
		}
  
		private class PoisonTimer : Timer 
		{ 
			private Mobile m_Mobile; 
			private Mobile a_Mobile; 
			private int m_hits; 

			public PoisonTimer( Mobile mobile, Mobile attacker, int hits ) : base( TimeSpan.FromSeconds( 1.5 ) ) 
			{ 
				Priority = TimerPriority.FiftyMS; 
				m_Mobile = mobile;
				a_Mobile = attacker;
			} 

			protected override void OnTick() 
			{ 
				AntLion lion = m_Mobile as AntLion; 
                  	
				if ( lion != null )
				{
					if ( lion.Hidden)
					{
						//if it is burrowed, stop the poison attack
						Stop();
						return;
					}

					if ( a_Mobile != null && a_Mobile.Alive && m_Mobile.InLOS( a_Mobile ) ) 
					{ 
						a_Mobile.ApplyPoison( lion, Poison.Greater );
						lion.MovingEffect( a_Mobile, 0x36D4, 7, 0, false, false, 0x238, 0 ); 
						a_Mobile.FixedParticles( 0x374A, 10, 15, 5021, EffectLayer.Waist );
						a_Mobile.PlaySound( 0x474 ); 
						Stop(); 
					}
				}
			}
		}
		
		public override int GetAngerSound()
		{
			return 0x5A;
		}

		public override int GetIdleSound()
		{
			return 0x5A;
		}

		public override int GetAttackSound()
		{
			return 0x164;
		}

		public override int GetHurtSound()
		{
			return 0x187;
		}

		public override int GetDeathSound()
		{
			return 0x1BA;
		}

		public AntLion( Serial serial ) : base( serial )
		{
		}
		 
		private class InternalTimer : Timer 
		{ 
			private AntLion m_Ant; 
			private Mobile m_Comb; 
			private Point3D m_Locale; 
			private int ticktock;

			public InternalTimer( AntLion mobile, Mobile c, Point3D locale ) : base( TimeSpan.FromSeconds( 1.5 ),TimeSpan.FromSeconds( 1.5 ) ) 
			{ 
				Priority = TimerPriority.FiftyMS; 
				m_Ant = mobile; 
				m_Comb = c;
				m_Locale = locale;
				ticktock = 0;      
			} 

			protected override void OnTick() 
			{
				if( m_Ant != null )
				{
					if ( ticktock == 0 ) 
					{ 
						if ( m_Comb != null && m_Comb.Alive )
						{
							//create a hole
							Item hole = new AntLionHole();
							m_Ant.m_hole1 = hole;
							hole.Map = m_Ant.Map;
							hole.Location = m_Ant.Location;
							m_Ant.PlaySound(546); //crumble
							m_Ant.Hidden = true;
							m_Ant.Frozen = true;
							m_Ant.Z -= 25; //put it "under the ground"...so you don't "push through invisible"
						}
						else
						{
							//m_Ant.Location = m_Locale;
							Stop();
						}
					}
					else if ( ticktock == 1 )
					{
						m_Ant.Hits += 10; // heal
					}
					else if ( ticktock == 2 )
					{
						m_Ant.Hits += 10; // heal
						if(m_Ant.m_hole1 != null)
							m_Ant.m_hole1.Delete(); //delete the hole
					}
					else if ( ticktock==3 ) 
					{ 
						//create a hole
						Item hole = new AntLionHole();
						m_Ant.m_hole2 = hole;
						hole.Map = m_Comb.Map;
						m_Ant.Location = m_Locale;
						
						//Per OSI, if the player is out of range, then it comes up in previous spot													
						if ( m_Comb != null && m_Comb.Alive && m_Ant.InRange( m_Comb, 10 ) )
						{
							m_Ant.Location = m_Comb.Location;
							hole.Location = m_Comb.Location;
							//add a little extra damage...
							m_Comb.DoHarmful( m_Ant ); 
							AOS.Damage( m_Comb, m_Ant, (int)Utility.Random(20,30), 100, 0, 0, 0, 0 );
						}
						else
						{
							hole.Location = m_Locale;
						}

						m_Ant.Hidden = false;
						m_Ant.Frozen = false;
						//eye candy...
						m_Ant.Say("*The ant lion bursts from the ground*");
						m_Ant.PlaySound(755);
						m_Ant.OnActionCombat();	//force this, so it doesn't stand there grinning moronicly										
					}
					else if ( ticktock == 4 )
					{	
						//nothing
					}
					else if ( ticktock == 5 )
					{
						//delete exit hole and stop timer
						if(m_Ant.m_hole2!=null)
							m_Ant.m_hole2.Delete();
						this.Stop();
					} 
					ticktock += 1;
				}
			}
		}
		
		public override bool OnBeforeDeath()
		{
			if(this.Frozen || this.Hidden)
			{
				this.Z += 25;
				this.Frozen = false;
				this.Hidden = false;
			}
			if(m_hole1!=null)
				m_hole1.Delete();
			if(m_hole2!=null)
				m_hole2.Delete();
			return true;
		}
			
		public override void OnAfterDelete()
		{
			if(m_hole1!=null)
				m_hole1.Delete();
			if(m_hole2!=null)
				m_hole2.Delete();
			
			base.OnAfterDelete();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 );				
			writer.Write( (Item) m_hole1 );
			writer.Write( (Item) m_hole2 );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();
			m_hole1 = reader.ReadItem();
			m_hole2 = reader.ReadItem();
		}
	}
}
