using System;
using System.Collections;
using Server;

namespace Server.Items
{
	public class MultiItemComponent : Item
	{
		private Point3D m_Offset;
		private MultiItem m_Base;

		[CommandProperty( AccessLevel.GameMaster )]
		public MultiItem Base
		{
			get
			{
				return m_Base;
			}
			set
			{
				m_Base = value;
			}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Point3D Offset
		{
			get
			{
				return m_Offset;
			}
			set
			{
				m_Offset = value;
			}
		}
		
		[Constructable]
		public MultiItemComponent( int itemID ) : base( itemID )
		{
			Movable = false;
		}

		public MultiItemComponent( Serial serial ) : base( serial )
		{
		}

		public override void OnLocationChange( Point3D old )
		{
			if ( m_Base != null )
				m_Base.Location = new Point3D( X - m_Offset.X, Y - m_Offset.Y, Z - m_Offset.Z );
		}

		public override void OnMapChange()
		{
			if ( m_Base != null )
				m_Base.Map = Map;
		}

		public override void OnAfterDelete()
		{
			base.OnAfterDelete();

			if ( m_Base != null )
				m_Base.Delete();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.Write( m_Base );
			writer.Write( m_Offset );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Base = reader.ReadItem() as MultiItem;
					m_Offset = reader.ReadPoint3D();

					if ( m_Base != null )
						m_Base.OnComponentLoaded( this );

					break;
				}
			}
		}
	}	

	public abstract class MultiItem : Item
	{
		private ArrayList m_Components;
		
		public void AddComponent( MultiItemComponent c, string name, int hue, int x, int y, int z )
		{
			if ( Deleted )
				return;

			m_Components.Add( c );

			c.Name = name;
			c.Hue = hue;
			c.Base = this;
			c.Offset = new Point3D( x, y, z );
			c.MoveToWorld( new Point3D( X + x, Y + y, Z + z ), Map );
		}

		public MultiItem() : base( 1 )
		{
			Movable = false;
			Visible = false;

			m_Components = new ArrayList();
		}
		
		public MultiItem( Serial serial ) : base( serial )
		{
		}

		public virtual void OnComponentLoaded( MultiItemComponent c )
		{
		}

		public override void OnLocationChange( Point3D oldLoc )
		{
			if ( Deleted )
				return;

			foreach ( MultiItemComponent c in m_Components )
				c.Location = new Point3D( X + c.Offset.X, Y + c.Offset.Y, Z + c.Offset.Z );
		}

		public override void OnMapChange()
		{
			if ( Deleted )
				return;

			foreach ( MultiItemComponent c in m_Components )
				c.Map = Map;
		}

		public override void OnAfterDelete()
		{
			base.OnAfterDelete();

			foreach ( MultiItemComponent c in m_Components )
				c.Delete();
		}
	
		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.WriteItemList( m_Components );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Components = reader.ReadItemList();
					break;
				}
			}
		}
	}

}
