using System; 
using System.Collections; 
using Server;
using Server.Misc;
using Server.Items; 
using Server.Mobiles; 
using Server.Gumps; 

namespace Server.Misc
{
	public abstract class BaseQuest : Item
	{
		private PlayerMobile m_Player;
		private ArrayList m_Objectives;
		private static TimeSpan QuestDelay = TimeSpan.FromMinutes( 60.0 );

		public BaseQuest( PlayerMobile player ) : base( 0x0 )
		{
			m_Player = player;	// Player that does the quest.
			m_Objectives = new ArrayList( 9 );
		}

		public BaseQuest( Serial serial ) : base( serial )
		{
		}

		// Basic quest properties
		// Localized version
		public virtual int QuestNameNumber{ get{ return 0; } }
		public virtual int OfferNumber{ get{ return 0; } }
		public virtual int StartNumber{ get{ return 0; } }
		public virtual int FinishNumber{ get{ return 0; } }
		public virtual int InProgressNumber{ get{ return 0; } }
		public virtual int LogObjectiveNumber{ get{ return 0; } }
		public virtual int LogStartObjectiveNumber{ get{ return 0; } }
		public virtual int LogFinishObjectiveNumber{ get{ return 0; } }
		public virtual int OverLoadedNumber{ get{ return 0; } }
	
		// String version
		public virtual string QuestNameString{ get { return Name; } }
		public virtual string OfferString{ get{ return ""; } }
		public virtual string StartString{ get{ return ""; } }
		public virtual string FinishString{ get{ return ""; } }
		public virtual string InProgressString{ get{ return ""; } }
		public virtual string LogObjectiveString{ get{ return ""; } }
		public virtual string LogStartObjectiveString{ get{ return ""; } }
		public virtual string LogFinishObjectiveString{ get{ return ""; } }
		public virtual string OverLoadedString{ get{ return ""; } }

		// General properties
		public virtual int QuestAcceptSound{ get{ return 0; } }
		public virtual bool Restartable{ get{ return true; } }
		public ArrayList Objectives{ get{ return m_Objectives; } }

		public PlayerMobile Player
		{
			get{ return m_Player; }
		}

		// Basic methods called by the gumps
		public virtual void Start()
		{
		}

		public virtual void Stop()
		{
			m_Player.CurrentQuest = null;
			m_Player.m_QuestRestartTime = (DateTime)( DateTime.Now + QuestDelay );
			
			foreach( QuestObjective objective in Objectives )
			{
				objective.Delete();
			}
			
			this.Delete();
		}

		public virtual void Cancel()
		{
			m_Player.CurrentQuest = null;
			
			foreach( QuestObjective objective in Objectives )
			{
				objective.Delete();
			}
			
			this.Delete();
		}

		public virtual void Finish()
		{
			Stop();
		}

		public virtual bool Completed
		{
			get
			{
				foreach( QuestObjective objective in m_Objectives )
				{
					if( !objective.Completed )
						return false;
				}
				return true;
			}
		}

		public virtual int CompletedObjectives
		{
			get
			{
				int amount = 0;

				foreach( QuestObjective objective in m_Objectives )
				{
					if( objective.Completed )
						amount++;
				}
				return amount;
			}
		}
		
		public virtual int TotalObjectives
		{
			get
			{
				int amount = 0;

				foreach( QuestObjective objective in m_Objectives )
				{
					if( objective != null )
						amount++;
				}
				return amount;
			}
		}

		public virtual bool CheckTargetedItem( Object o, Item item, PlayerMobile player )
		{
			return false;
		}

		public virtual bool CanRecieveItem( PlayerMobile player, Item item )
		{
			Container bp = player.Backpack;
			
			if ( bp == null )
			{
				item.Delete();
				return false;
			}
			
			if( bp.TryDropItem( player, item, false ) )
				return true;
				
			item.Delete();
			return false;
		}

		public override string ToString()
		{
			return "Quest";
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 ); // version

			writer.Write( (Mobile) m_Player );
			writer.WriteItemList( m_Objectives );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();

			m_Player = (PlayerMobile) reader.ReadMobile();
			m_Objectives = reader.ReadItemList();
		}
	}

	public class QuestObjective : Item
	{
		private static int m_DescriptionNumber;
		private static string m_DescriptionString;

		private bool m_Completed;

		public QuestObjective( int description ) : base( 0x0 )
		{
			m_DescriptionNumber = description;
			m_DescriptionString = "";
			m_Completed = false;
		}

		public QuestObjective( string description ) : base( 0x0 )
		{
			m_DescriptionNumber = 0;
			m_DescriptionString = description;
			m_Completed = false;
		}

		public QuestObjective( Serial serial ) : base( serial )
		{
		}

		public bool Completed
		{
			get{ return m_Completed; }
			set{ m_Completed = value; }
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );
			writer.Write( (int) 0 ); // version

			writer.Write( (int) m_DescriptionNumber );
			writer.Write( (string) m_DescriptionString );
			writer.Write( (bool) m_Completed );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );
			int version = reader.ReadInt();

			m_DescriptionNumber = reader.ReadInt();
			m_DescriptionString = reader.ReadString();
			m_Completed = reader.ReadBool();
		}
	}
}